function buildCompGraph(obj)

    fprintf("Building Computational Graph.\n");
    
    obj.ndOrigTomoVol = comg.Node("Original Tomogram Volume", []);
    obj.ndStatFlipContrast = comg.Node("Whether flip tomo contrast", false);

    obj.ndTomoVol = comg.Node.derived("TomoVol", ...
        @(tomovol, flag)((0.5-flag)*2*tomovol), ...
        obj.ndOrigTomoVol, obj.ndStatFlipContrast);

    obj.ndTomoLayInd = comg.Node("TomoVolInd", []);
    obj.ndFilaEngn = comg.Node("FilamentEngine", []);

    obj.ndTFStat = comg.Node("TF Status", obj.tf_stat_free);
    obj.ndStatTFFree = comg.Node.derived("In Free Status", ...
        @(stat)(stat == obj.tf_stat_free), ...
        obj.ndTFStat).activate();
    obj.ndStatTFAdd = comg.Node.derived("In Adding Status", ...
        @(stat)(stat == obj.tf_stat_adding), ...
        obj.ndTFStat).activate();
    obj.ndStatTFMov = comg.Node.derived("In Moving Status", ...
        @(stat)(stat == obj.tf_stat_moving), ...
        obj.ndTFStat).activate();
    obj.ndStatTFFoc = comg.Node.derived("In Focusing Status", ...
        @(stat)(stat == obj.tf_stat_focusing), ...
        obj.ndTFStat).activate();
    obj.ndStatTFShowCurrPt = comg.Node.derived("Show Current Point", ...
        @(foc, add, mov)(~foc && ~add && ~mov), ...
        obj.ndStatTFFoc, obj.ndStatTFAdd, obj.ndStatTFMov).activate();
    obj.ndStatTFFTVa = comg.Node.derived("In Setting Va", ...
        @(stat)(stat == obj.tf_stat_setting_va), ...
        obj.ndTFStat).activate();
    obj.ndStatTFFTVb = comg.Node.derived("In Setting Vb", ...
        @(stat)(stat == obj.tf_stat_setting_vb), ...
        obj.ndTFStat).activate();
    obj.ndStatTFFTVs = comg.Node.derived("In Setting Vb or Vb", ...
        @or, ...
        obj.ndStatTFFTVa, obj.ndStatTFFTVb).activate();

    obj.ndStatSpecRmvBg = comg.Node("Removing background in Spectrum", false);
    obj.ndStatSurfSymmAvg = comg.Node("Symmetrically Averaging Surfaces", false);
    
    obj.ndFilaSecInd = comg.Node("Current Filament Section Index", 1);
    obj.ndFilaSecAvgNum = comg.Node("Filament Section Averaging Number", 10);
    obj.ndFilaInnRad = comg.Node("Filament Inner Radius", 5);
    obj.ndFilaOutRad = comg.Node("Filament Outer Radius", 10);
    obj.ndFilaPolSco = comg.Node("Polarity Scores", 0);

    obj.ndExtrFila = comg.Node("Extracted Filament", []);
    obj.ndExtrFilaRect = comg.Node("Extracted Filament (Rectangle)", []);
    obj.ndExtrFilaPts = comg.Node("Extracted Filament Points", []);
    obj.ndExtrFilaSmthPts = comg.Node("Extracted Filament Smoothed Points", []);
    obj.ndExtrFilaPtLens = comg.Node("Extracted Filament Point Length", []);
    obj.ndExtrFilaRMFr = comg.Node("Extracted Filament RMF r", []);
    obj.ndExtrFilaRMFs = comg.Node("Extracted Filament RMF s", []);
    obj.ndExtrFilaRMFt = comg.Node("Extracted Filament RMF t", []);
    
    obj.ndSpecSegSize = comg.Node("Segment Size for Calculating Spectrum", 100);
    obj.ndSpecSegStep = comg.Node("Segment Step for Calculating Spectrum", 50);
    obj.ndSpecGamma = comg.Node("Gamma for Showing Spectrum", 0);
    obj.ndSpecShowOrd = comg.Node("Max Order of Showing Spectrum", 30);
    obj.ndSpecOptLen = comg.Node("Optimization Range at Length Direction", 2);
    obj.ndSpecOptAng = comg.Node("Optimization Range at Angle Direction", 2);
    
    obj.ndSpecVa = comg.Node("Reciprocal Vector A", []);
    obj.ndSpecVb = comg.Node("Reciprocal Vector B", []);
    obj.ndSpecVa.updateValue([]);
    obj.ndSpecVb.updateValue([]);

    obj.ndTFNewFilaPts = comg.Node("New Filament Points", []);
    obj.ndTFNewFilaPts.updateValue([]);

    obj.ndTFFocPtInd = comg.Node("Focusing point ind", []);
    obj.ndTFFocPtInd.updateValue([]);

    obj.ndTFTracThr = comg.Node("Tracing Threshold", 0.05);

    obj.ndTFCurrPt = comg.Node("Current Point", []);
    obj.ndTFCurrPt.updateValue([]);
    obj.ndTFCurrPtSpc = comg.Node("Current Point Space", "Eul");

    obj.ndCrtFilaPts = comg.Node.derived("CreatingFilamentPoints", ...
        @(fila_engine)(fila_engine.getCreatingFilamentPoints()), ...
        obj.ndFilaEngn);

    obj.ndSelFilaPts = comg.Node.derived("SelectingFilamentPoints", ...
        @getSelectingFilamentPoints, ...
        obj.ndFilaEngn);

    obj.ndAllFilaPts = comg.Node.derived("AllFilamentPoints", ...
        @getAllFilamentPoints, ...
        obj.ndFilaEngn);

    obj.ndTunFocPt = comg.Node.derived("TuningFilamentFocusingPoints", ...
        @TF_getFocusingPoint, ...
        obj.ndFilaEngn);

    obj.ndShowSurf = comg.Node.derived("Surface to Show", ...
        @drvShowSurf, ...
        obj.ndExtrFila, obj.ndFilaInnRad, obj.ndFilaOutRad, obj.ndStatSurfSymmAvg, obj.ndSpecVa, obj.ndSpecVb);

    obj.ndCylSpec = comg.Node.derived("Cylindrical Spectrum", ...
        @drvCylSpec, ...
        obj.ndExtrFila, obj.ndFilaInnRad, obj.ndFilaOutRad, obj.ndSpecSegSize, obj.ndSpecSegStep);
    
    obj.ndLogSpec = comg.Node.derived("Log Cylindrical Spectrum", ...
        @(spec, gamma)(log(1+10.^(-gamma)*spec)), ...
        obj.ndCylSpec, obj.ndSpecGamma);

    obj.ndTylLogSpec = comg.Node.derived("Taylored Log Spectrum", ...
        @drvTylLogSpec, ...
        obj.ndLogSpec, obj.ndStatSpecRmvBg, obj.ndSpecShowOrd);

    obj.ndStatRpvValid = comg.Node.derived("Is Reci Primi Vectors Valid", ...
        @helix.rpvValid, ...
        obj.ndSpecVa, obj.ndSpecVb);

    obj.ndHp = comg.Node.derived("Helical Parameters", ...
        3, @helix.rpv2hp, ...
        obj.ndSpecVa, obj.ndSpecVb);

    obj.ndHpSn = obj.ndHp.subnode("Start Number", 1);
    obj.ndHpRise = obj.ndHp.subnode("Rise", 2);
    obj.ndHpTwist = obj.ndHp.subnode("Twist", 3);

    obj.ndTFCurrPtCds = comg.Node.derived("", ...
        2, @drvPointCoord, ...
        obj.ndTFCurrPt, obj.ndTFCurrPtSpc, obj.ndExtrFilaSmthPts, obj.ndExtrFilaRMFr, obj.ndExtrFilaRMFs, obj.ndExtrFilaRMFt);

    comg.Node.output(@(cd,sp)(fprintf("Current Point is (%8.3f, %8.3f, %8.3f), in %s space.\n", cd, sp)), ...
        obj.ndTFCurrPt, obj.ndTFCurrPtSpc).activate();

    obj.ndTFCurrPtEul = obj.ndTFCurrPtCds.subnode("Current Point in Euler", 1);
    obj.ndTFCurrPtCyl = obj.ndTFCurrPtCds.subnode("Current Point in Cylin", 2);

    comg.Node.output(@(cd,sp)(fprintf("Current Point Euler coords is (%8.3f, %8.3f, %8.3f).\n", cd)), ...
        obj.ndTFCurrPtEul).activate();
    comg.Node.output(@(cd,sp)(fprintf("Current Point Cylin coords is (%8.3f, %8.3f, %8.3f).\n", cd)), ...
        obj.ndTFCurrPtCyl).activate();

    obj.ndTFFocPt = comg.Node.derived("TF Focusing Point", ...
        @drvTFFocPt, ...
        obj.ndTFNewFilaPts, obj.ndTFFocPtInd);

    ndNewFilaPointConverter = comg.Node.derived("Converted Filament Points", ...
        2, @drvNewFilaPtLens, ...
        obj.ndTFNewFilaPts, obj.ndExtrFilaSmthPts, obj.ndExtrFilaRMFr, obj.ndExtrFilaRMFs, obj.ndExtrFilaRMFt);
    ndNewFilaPointConverter.activate();

    obj.ndTFNewFilaPtsInSec = ndNewFilaPointConverter.subnode(1);
    obj.ndTFNewFilaPtLens = ndNewFilaPointConverter.subnode(2);
    
    obj.ndTFFocPtSecInd = comg.Node.derived("Focusing Point Section Index", ...
        @drvTFFocPtSecInd, ...
        obj.ndTFNewFilaPtLens, obj.ndTFFocPtInd).activate();

    obj.ndTFNewFilaSmthPts = comg.Node.derived("New Smoothed Filament Points", ...
        @helix.filamentSmoothing, ...
        obj.ndTFNewFilaPts);

    ndNewFilaSmthPointConverter = comg.Node.derived("Converted Filament Points", ...
        2, @drvNewFilaPtLens, ...
        obj.ndTFNewFilaSmthPts, obj.ndExtrFilaSmthPts, obj.ndExtrFilaRMFr, obj.ndExtrFilaRMFs, obj.ndExtrFilaRMFt);
    ndNewFilaSmthPointConverter.activate();

    obj.ndTFNewFilaSmthPtsInSec = ndNewFilaSmthPointConverter.subnode(1);
    obj.ndTFNewFilaSmthPtLens = ndNewFilaSmthPointConverter.subnode(2);

    obj.ndViewLattPt = comg.Node("Flag: Show Lattice Points", false).activate();
    obj.ndViewLattLine = comg.Node("Flag: Show Lattice Lines", false).activate();
    obj.ndViewLattBspt = comg.Node("Lattice meshgrid base point", [0;0]).activate();
    
end

function tylspec = drvTylLogSpec(logspec, flag_rmvbg, ord)
    if flag_rmvbg
        bg = movmedian(movmedian(logspec, 15, 1), 15, 2);
        logspec = logspec - bg;
    end
    tylspec = cenrsz(logspec, [size(logspec, 1), ord*2+1]);
end

function showsurf = drvShowSurf(extr_fila, innrad, outrad, flag_symavg, rpva, rpvb)
    if isempty(extr_fila)
        showsurf = [];
        return;
    end
    showsurf = sum(extr_fila(:,:,innrad:outrad),3);
    if flag_symavg
        showsurf = helix.surfHelicalAveraging(showsurf, rpva, rpvb);
    end
end

function [eulcd, cylcd] = drvPointCoord(coord, spc, smthpts, RMFr, RMFs, RMFt)
    if spc == helix.TomoFilamentUIManager.space_euler
        eulcd = coord;
        cylcd = helix.ptEul2Cyl(smthpts, RMFr, RMFs, RMFt, eulcd);
    elseif spc == helix.TomoFilamentUIManager.space_cylin
        cylcd = coord;
        eulcd = helix.ptCyl2Eul(smthpts, RMFr, RMFs, RMFt, coord);
    end
end

function [insec, newlens] = drvNewFilaPtLens(coord, smthpts, RMFr, RMFs, RMFt)
    if isempty(coord)||isempty(smthpts)||isempty(RMFr)||isempty(RMFs)||isempty(RMFt)
        insec = [];
        newlens = [];
        return;
    end
    cylcd = helix.ptEul2Cyl(smthpts, RMFr, RMFs, RMFt, coord);
    insec = cylcd(1:2, :);
    newlens = cylcd(3, :);
end

function cylspec = drvCylSpec(extr_fila, innrad, outrad, segsz, segst)
    if ~isempty(extr_fila)
        cylspec = helix.segmentSpectrum(extr_fila(:,:,innrad:outrad), segsz, segst);
    else
        cylspec = [];
    end
end

function focsecind = drvTFFocPtSecInd(ptlens, focind)
    if isempty(ptlens) || isempty(focind)
        focsecind = [];
        return;
    end
    focsecind = floor(ptlens(focind)) +1;
end

function focpt = drvTFFocPt(pts, ind)
    if isempty(pts)
        focpt = [];
        return;
    end
    focpt = pts(:,ind);
end

