function registerCylinAxes(obj, ax)
    initAxes(ax);

    click_callback = @(src, evt)(obj.cylinClickedCallback(src, evt));
    ndShowSurf = comg.Node.output("Show Filament Unrolled Surface", ...
            @(showsurf, inn_rad, out_rad)(showFilaSurf(ax, click_callback, showsurf, inn_rad, out_rad, obj.extr_ref_theta)), ...
            obj.ndShowSurf, obj.ndFilaInnRad, obj.ndFilaOutRad ...
        ).activate();

    comg.Node.output("Draw Sample Points in Filament Unrolled Surface", ...
            @(extr_fila_ptlens, out_rad)(showFilaSamplePoints(ax, extr_fila_ptlens, out_rad, obj.extr_ref_theta)), ...
            obj.ndExtrFilaPtLens, obj.ndFilaOutRad ...
        ).activate().after(ndShowSurf);

    comg.Node.output("Draw Selected Section in Filament Unrolled Surface", ...
            @(secind, out_rad)(showSelectedSec(ax, secind, out_rad, obj.extr_ref_theta)), ...
            obj.ndFilaSecInd, obj.ndFilaOutRad ...
        ).activate().after(ndShowSurf);

    comg.Node.output("Draw New Filament Points in Unrolled Surface", ...
            @(fila_ptlens, out_rad, flag_foc, foc_ind)(showNewFilaPoints(ax, fila_ptlens, out_rad, flag_foc, foc_ind, obj.extr_ref_theta)), ...
            obj.ndTFNewFilaPtLens, obj.ndFilaOutRad, obj.ndStatTFFoc, obj.ndTFFocPtInd ...
        ).activate().after(ndShowSurf);

    comg.Node.output("Draw Current Point Unrolled Surface", ...
            @(currptcyl, out_rad, flag_show)(showCurrPoint(ax, currptcyl, out_rad, obj.extr_ref_theta, flag_show)), ...
            obj.ndTFCurrPtCyl, obj.ndFilaOutRad, obj.ndStatTFShowCurrPt ...
        ).activate().after(ndShowSurf);

    comg.Node.output("Draw Lattice Point", ...
            @(out_rad, va, vb, bp, flag_show)(showLattPoint(ax, out_rad, obj.extr_ref_theta, va, vb, bp, flag_show)), ...
            obj.ndFilaOutRad, obj.ndSpecVa, obj.ndSpecVb, obj.ndViewLattBspt, obj.ndViewLattPt ...
        ).activate().after(ndShowSurf);

    comg.Node.output("Draw Lattice Line", ...
            @(out_rad, va, vb, bp, flag_show)(showLattLine(ax, out_rad, obj.extr_ref_theta, va, vb, bp, flag_show)), ...
            obj.ndFilaOutRad, obj.ndSpecVa, obj.ndSpecVb, obj.ndViewLattBspt, obj.ndViewLattLine ...
        ).activate().after(ndShowSurf);

end

function initAxes(ax)
    xlabel(ax, "Arc Length");
    ylabel(ax, "Length");
    colormap(ax, 'gray');
    hold(ax, "on");
    axis(ax, "equal");
    title(ax, "Surface");
end

function showFilaSurf(ax, click_callback, showsurf, inn_rad, out_rad, extr_ref_theta)
    persistent handle_filasurf;

    delete(handle_filasurf);
    if ~isempty(showsurf)
        handle_filasurf = imagesc(ax,...
            extr_ref_theta/180*pi*out_rad,...
            0:(size(showsurf, 1)-1),...
            showsurf,...
            "ButtonDownFcn", click_callback);
        axis(ax, "equal");
        xlim(ax, (extr_ref_theta([1,end])+[-50,50])/180*pi*out_rad);
        ylim(ax, [0,(size(showsurf, 1)-1)]);
    end
end

function showFilaSamplePoints(ax, extr_fila_ptlens, out_rad, extr_ref_theta)
    persistent handle_ptlines;
    color = [.8, .4, .4];

    for ii = 1:numel(handle_ptlines)
        delete(handle_ptlines{ii});
    end
    num_pts = numel(extr_fila_ptlens);
    handle_ptlines = cell(1, num_pts);
    for ii = 1:num_pts
        ptl = extr_fila_ptlens(ii);
        handle_ptlines{ii} = plot(ax, ...
            (extr_ref_theta(1)+[-50,0])/180*pi*out_rad, [ptl, ptl], ...
            'Color', color,...
            'LineWidth', 2,...
            'PickableParts', 'none');
    end
end

function showSelectedSec(ax, secind, out_rad, extr_ref_theta)
    persistent handle_secline;
    color = [.4, .6, .6];

    delete(handle_secline);
    sec_len = secind - 1;
    handle_secline = plot(ax, ...
        (extr_ref_theta([1,end])+[-30,30])/180*pi*out_rad, [sec_len, sec_len], ...
        'Color', color,...
        'LineWidth', 2,...
        'PickableParts', 'none');
end

function showNewFilaPoints(ax, fila_ptlens, out_rad, flag_foc, foc_ind, extr_ref_theta)
    persistent handle_ptlines;
    common_color = [.6, .6, .4];
    foc_color = [.4, .8, .4];

    for ii = 1:numel(handle_ptlines)
        delete(handle_ptlines{ii});
    end
    num_pts = numel(fila_ptlens);
    handle_ptlines = cell(1, num_pts);
    for ii = 1:num_pts
        if flag_foc && (ii == foc_ind)
            color = foc_color;
        else
            color = common_color;
        end
        ptl = fila_ptlens(ii);
        handle_ptlines{ii} = plot(ax, ...
            (extr_ref_theta(end)+[0,50])/180*pi*out_rad, [ptl, ptl], ...
            'Color', color,...
            'LineWidth', 2,...
            'PickableParts', 'none');
    end
end

function showCurrPoint(ax, currptcyl, out_rad, extr_ref_theta, flag_show)
    persistent handle_secline;
    color = [.4, .4, .8];

    delete(handle_secline);
    if flag_show && ~isempty(currptcyl)
        sec_len = currptcyl(3) - 1;
        handle_secline = plot(ax, ...
            (extr_ref_theta([1,end])+[-30,30])/180*pi*out_rad, [sec_len, sec_len], ...
            'Color', color,...
            'LineWidth', 2,...
            'PickableParts', 'none');
    end

end

function showLattPoint(ax, out_rad, extr_ref_theta, va, vb, base_pt, flag_show)
    persistent handle_latt_pt;
    color = art.color.Ambers{4};

    delete(handle_latt_pt);
    if flag_show && ~isempty(va) && ~isempty(vb) && ~isempty(out_rad)
        [ua, ub] = helix.rpv2pv(va, vb);
        latt_pt = helix.latticePoints(ua, ub, extr_ref_theta([1,end]), ax.YLim, [base_pt(1)/out_rad/pi*180;base_pt(2)]);
        handle_latt_pt = plot(ax, latt_pt(1,:)/180*pi*out_rad, latt_pt(2,:), ...
            'LineStyle', 'none', ...
            'Marker', '.', ...
            'Color', color, ...
            'PickableParts', 'none');
    end

end

function showLattLine(ax, out_rad, extr_ref_theta, va, vb, base_pt, flag_show)
    persistent handle_latt_line;
    color = art.color.Oranges{4};

    for ii = 1:numel(handle_latt_line)
        delete(handle_latt_line{ii});
    end
    if flag_show && ~isempty(va) && ~isempty(vb) && ~isempty(out_rad)
        [ua, ub] = helix.rpv2pv(va, vb);
        [stpts, edpts] = helix.latticeLines(ua, ub, extr_ref_theta([1,end]), ax.YLim, [base_pt(1)/out_rad/pi*180;base_pt(2)]);
        
        handle_latt_line = cell(1, size(stpts, 2));
        for ii = 1:size(stpts, 2)
            handle_latt_line{ii} = plot(ax, [stpts(1,ii), edpts(1,ii)]/180*pi*out_rad, [stpts(2,ii), edpts(2,ii)], ...
                'LineStyle', '-', ...
                'LineWidth', 1, ...
                'Marker', 'none', ...
                'Color', color, ...
                'PickableParts', 'none');
        end
    end

end
