function registerTFFilaTraceButton(obj, butt)
    arguments
        obj helix.TomoFilamentUIManager
        butt (1,1) matlab.ui.control.Button
    end

    setContexMenu(butt, obj);

    butt.ButtonPushedFcn = @(src, evt)(buttonPushedCallback( ...
        src, evt, obj));
 
    comg.Node.output("Enable Filament-Tracing-Button",...
            @(fila_engine, stat_free, stat_foc)(enableComponent(butt, fila_engine, stat_free, stat_foc)), ...
            obj.ndFilaEngn, obj.ndStatTFFree, obj.ndStatTFFoc ...
        ).activate();
end

function setContexMenu(butt, obj)
    arguments
        butt (1,1) matlab.ui.control.Button
        obj (1,1) helix.TomoFilamentUIManager
    end

    fig = butt.Parent;
    while ~isa(fig, "matlab.ui.Figure")
        fig = fig.Parent;
    end

    context_menu = uicontextmenu(fig);

    set_parameters_menu = uimenu(context_menu);
    set_parameters_menu.Text = "Set tracing parameters";
    set_parameters_menu.MenuSelectedFcn = @(src, evt)(menuSelectedCallback(src, evt,obj));
    
    butt.ContextMenu = context_menu;
end

function menuSelectedCallback(src, evt, obj)
    fprintf("Set parameter menu Selected.\n");
    
    dlg_res = inputdlg(obj.fila_trace_paras.keys,"Filament Tracing Parameters", 1, ...
        obj.fila_trace_paras.values);
    allkeys = obj.fila_trace_paras.keys;
    for ii = 1:obj.fila_trace_paras.Count
        obj.fila_trace_paras(string(allkeys(ii))) = string(dlg_res(ii));
    end

end

function enableComponent(component, fila_engine, stat_free, stat_foc)
    flag_enable = fila_engine.inFilamentTuningMode();
    flag_enable = flag_enable && (stat_free || stat_foc);

    component.Enable = flag_enable;
end

function buttonPushedCallback(src, evt, obj)
    fprintf("Filament Tracing Button Pressed.\n");

    tomo_vol = obj.ndTomoVol.value;
    init_pts = obj.ndTFNewFilaPts.value;

    templ_rad = str2double(obj.fila_trace_paras("template radius (pix)"));
    templ_len = str2double(obj.fila_trace_paras("template length (pix)"));
    trace_step = str2double(obj.fila_trace_paras("tracing step (pix)"));
    trace_numb = str2double(obj.fila_trace_paras("tracing point number"));
    sear_step = str2double(obj.fila_trace_paras("searching step (pix)"));
    sear_num = str2double(obj.fila_trace_paras("searching number"));
    border_dist = [1,1,1] * str2double(obj.fila_trace_paras("border reserve distance"));
    score_thres = obj.ndTFTracThr.value;
    
    [full_fila_pts, ~, ~] = helix.filamentTracing3D( ...
        tomo_vol, init_pts, templ_rad, templ_len, trace_step, sear_step, sear_num, ...
        'arc_approx', false, 'score_thres', score_thres, 'border_dist', border_dist, ...
        'max_pt_num', trace_numb);

    obj.ndTFNewFilaPts.updateValue(full_fila_pts);
end

