function registerTFPolScoButton(obj, butt)
    arguments
        obj helix.TomoFilamentUIManager
        butt matlab.ui.control.Button
    end

    butt.ButtonPushedFcn = @(src, evt)(buttonPushedCallback(obj, src, evt));
    
    comg.Node.output("Enable PolSco Button",...
            @(fila_engine)(enableComponent(butt, fila_engine)), obj.ndFilaEngn ...
        ).activate();
end

function enableComponent(component, fila_engine)
    flag_enable = fila_engine.inFilamentTuningMode();

    component.Enable = flag_enable;
end

function buttonPushedCallback(obj, src, evt)
    arguments
        obj helix.TomoFilamentUIManager
        src 
        evt 
    end

    fprintf("Polarity Score button pushed.\n");

    [ref_file, sfsta_file] = mrc_file_inputdlg();
    if ~isempty(ref_file) & ~isempty(sfsta_file)
        refmap = readmrc(ref_file);
        stamap = readmrc(sfsta_file);
        refmap = cenrsz(refmap, [obj.ndFilaOutRad.value*2 + 5, obj.ndFilaOutRad.value*2 + 5, round(size(refmap, 3)/2)]);
        stamap = cenrsz(stamap, [obj.ndFilaOutRad.value*2 + 5, obj.ndFilaOutRad.value*2 + 5, size(refmap, 3)]);
        score = helix.determinePolarityScore(refmap, stamap);
        obj.ndFilaPolSco.updateValue(score);
    end

    fprintf("Done.\n");
end

function [ref_file, sfsta_file] = mrc_file_inputdlg()
    ref_file = '';
    sfsta_file = '';

    d = dialog('Position',[400 400 500 220],'Name','Select .mrc files');

    uicontrol('Parent',d,'Style','text','Position',[20 170 100 20],'String','Reference map:','HorizontalAlignment','left');
    editA = uicontrol('Parent',d,'Style','edit','Position',[120 170 280 25],'HorizontalAlignment','left','String','');
    uicontrol('Parent',d,'Position',[410 170 70 25],'String','Scan...','Callback', @(~,~) onBrowse(editA));

    uicontrol('Parent',d,'Style','text','Position',[20 120 100 20],'String','SF-STA map:','HorizontalAlignment','left');
    editB = uicontrol('Parent',d,'Style','edit','Position',[120 120 280 25],'HorizontalAlignment','left','String','');
    uicontrol('Parent',d,'Position',[410 120 70 25],'String','Scan...','Callback', @(~,~) onBrowse(editB));

    uicontrol('Parent',d,'Position',[120 40 100 30],'String','Confirm','Callback', @onOK);
    uicontrol('Parent',d,'Position',[280 40 100 30],'String','Cancel','Callback', @onCancel);

    uiwait(d);

    function onBrowse(editHandle)
        [fname, fpath] = uigetfile('*.mrc', 'Select .mrc file');
        if fname ~= 0
            set(editHandle, 'String', fullfile(fpath, fname));
        end
    end

    function onOK(src, evt)
        ref_file = get(editA, 'String');
        sfsta_file = get(editB, 'String');
        delete(d);
    end

    function onCancel(src, evt)
        ref_file = '';
        sfsta_file = '';
        delete(d);
    end
end