function registerTuneFilaButton(obj, butt)
    butt.ButtonPushedFcn = @(src, evt)(buttonPushedCallback(obj.ndFilaEngn, ...
        obj.ndTFStat, src, evt, obj));
    
    setContexMenu(butt, obj);

    comg.Node.output("Enable Accept-Filament-Creating-Button",...
            @(fila_engine)(enableComponent(butt, fila_engine)), ...
            obj.ndFilaEngn ...
        ).activate();
end

function setContexMenu(butt, obj)
    arguments
        butt (1,1) matlab.ui.control.Button
        obj (1,1) helix.TomoFilamentUIManager
    end

    fig = butt.Parent;
    while ~isa(fig, "matlab.ui.Figure")
        fig = fig.Parent;
    end

    context_menu = uicontextmenu(fig);

    set_parameters_menu = uimenu(context_menu);
    set_parameters_menu.Text = "Set extracting parameters";
    set_parameters_menu.MenuSelectedFcn = @(src, evt)(menuSelectedCallback(src, evt,obj));
    
    butt.ContextMenu = context_menu;
end

function menuSelectedCallback(src, evt, obj)
    arguments
        src
        evt
        obj helix.TomoFilamentUIManager
    end

    fprintf("'Set extracting parameters' menu Selected.\n");
    
    field_names = fieldnames(obj.config.extract_parameters);
    init_vals = cellfun(@(fn)({num2str(obj.config.extract_parameters.(fn))}), field_names);

    dlg_res = inputdlg(field_names,"Filament Extracting Parameters", 1, ...
        init_vals);

    for ii = 1:numel(dlg_res)
        fn = field_names{ii};
        obj.config.extract_parameters.(fn) = str2double(dlg_res{ii});
    end

    obj.updateConfigurations();

end

function enableComponent(component, fila_engine)
    flag_enable = fila_engine.allowEnterFilamentTuningMode();
    component.Enable = flag_enable;
end

function buttonPushedCallback(ndFilaEngn, ndTFStat, src, evt, obj)
    arguments
        ndFilaEngn
        ndTFStat
        src
        evt
        obj helix.TomoFilamentUIManager
    end
    
    fprintf("Tune Filament button pushed.\n");

    ndFilaEngn.value.enterTuningFilamentMode();
    tag = ndFilaEngn.value.TF_getFilamentTag();
    upder = [];
    if isstruct(tag)
        if ismember("inner_rad", fieldnames(tag))
            upder = obj.ndFilaInnRad.updateValue(tag.inner_rad, upder);
        end
        if ismember("outer_rad", fieldnames(tag))
            upder = obj.ndFilaOutRad.updateValue(tag.outer_rad, upder);
        end
        if ismember("rpv_a", fieldnames(tag))
            upder = obj.ndSpecVa.updateValue(tag.rpv_a, upder);
        end
        if ismember("rpv_b", fieldnames(tag))
            upder = obj.ndSpecVb.updateValue(tag.rpv_b, upder);
        end
    end
    upder = ndFilaEngn.update(upder);
    ndTFStat.updateValue(helix.TomoFilamentUIManager.tf_stat_free, upder);
end






