function registerViewControlTree(obj, tree)
    arguments
        obj helix.TomoFilamentUIManager
        tree matlab.ui.container.CheckBoxTree
    end
    tree.CheckedNodesChangedFcn = @(src, evt)(checkedNodesChangedFcn(obj, src, evt));

    group_node_cylinsurf = uitreenode(tree);
    group_node_cylinsurf.Text = 'cylindrical surface';

    node_lattice_pt = uitreenode(group_node_cylinsurf);
    node_lattice_pt.Text = 'lattice points';

    node_lattice_line = uitreenode(group_node_cylinsurf);
    node_lattice_line.Text = 'lattice lines';

    setContexMenuForLattMesh(node_lattice_pt, node_lattice_line, obj);
end

function setContexMenuForLattMesh(node_pt, node_line, obj)
    arguments
        node_pt (1,1) matlab.ui.container.TreeNode
        node_line (1,1) matlab.ui.container.TreeNode
        obj (1,1) helix.TomoFilamentUIManager
    end

    fig = node_pt.Parent;
    while ~isa(fig, "matlab.ui.Figure")
        fig = fig.Parent;
    end

    context_menu = uicontextmenu(fig);

    set_parameters_menu = uimenu(context_menu);
    set_parameters_menu.Text = "Set lattice base point";
    set_parameters_menu.MenuSelectedFcn = @(src, evt)(menuSelectedCallback(obj, src, evt));
    
    node_pt.ContextMenu = context_menu;
    node_line.ContextMenu = context_menu;
end

function menuSelectedCallback(obj, src, evt)
    arguments
        obj helix.TomoFilamentUIManager
        src
        evt
    end

    para_tag_key_val = { ...
        'Base point position: arc length (pixel)', obj.ndViewLattBspt.value(1); ...
        'Base point position: height (pixel)', obj.ndViewLattBspt.value(2); ...
        };

    dlg_res = inputdlg(para_tag_key_val(:,1), "Lattice Meshgrid Parameters", 1, ...
        cellfun(@(xx)({num2str(xx)}),para_tag_key_val(:,2)));

    if ~isempty(dlg_res)
        bspt = zeros(2,1);
        bspt(1) = str2double(dlg_res{1});
        bspt(2) = str2double(dlg_res{2});
        obj.ndViewLattBspt.updateValue(bspt);
    end
end

function checkedNodesChangedFcn(obj, src, evt)
    arguments
        obj helix.TomoFilamentUIManager
        src matlab.ui.container.CheckBoxTree
        evt
    end

    changed_nodes = setxor(evt.PreviousCheckedNodes, evt.CheckedNodes);
    changed_texts = arrayfun(@(cnl)({cnl.Text}), changed_nodes);

    upd = [];
    if ismember('lattice points', changed_texts)
        upd = obj.ndViewLattPt.updateValue(~obj.ndViewLattPt.value);
    end
    if ismember('lattice lines', changed_texts)
        upd = obj.ndViewLattLine.updateValue(~obj.ndViewLattLine.value);
    end

    comg.Node.update_specific_nodes(upd);

end



