function [rect_extr, cylin_extr, RMF_r, RMF_s, RMF_t, axis_sample_coords]...
     = filamentExtracting(vol, curve, step, len, rad, ref_theta)

    axis_sample_coords = fnval(curve, 0:step:(len+step));
    [RMF_r, RMF_s, RMF_t] = calcRMF(axis_sample_coords, [0;0;1]);

    % RMF_r: x
    % RMF_s: y
    % RMF_t: z
    [plane_coord_x, plane_coord_y] = cencoord([1,1]*(1+2*floor(rad/step)), step);
    
    sample_points_x = plane_coord_x.*reshape(RMF_r(1,:),1,1,[]) + ...
                      plane_coord_y.*reshape(RMF_s(1,:),1,1,[]) + ...
                      reshape(axis_sample_coords(1,:),1,1,[]);
    sample_points_y = plane_coord_x.*reshape(RMF_r(2,:),1,1,[]) + ...
                      plane_coord_y.*reshape(RMF_s(2,:),1,1,[]) + ...
                      reshape(axis_sample_coords(2,:),1,1,[]);
    sample_points_z = plane_coord_x.*reshape(RMF_r(3,:),1,1,[]) + ...
                      plane_coord_y.*reshape(RMF_s(3,:),1,1,[]) + ...
                      reshape(axis_sample_coords(3,:),1,1,[]);
    
    vol_x = 1:size(vol, 1);
    vol_y = 1:size(vol, 2);
    vol_z = 1:size(vol, 3);

    rect_extr = interp3(vol_y, vol_x, vol_z, vol, sample_points_y, sample_points_x, sample_points_z, 'cubic', mean(vol, "all"));
    % rect_extr: RMF_r, RMF_s, L

    if nargout > 1
        ref_rad = 0:rad;
        plane_coord_x = ref_rad(:) .* cosd(ref_theta(:)');
        plane_coord_y = ref_rad(:) .* sind(ref_theta(:)');
        
        sample_points_x = plane_coord_x.*reshape(RMF_r(1,:),1,1,[]) + ...
                          plane_coord_y.*reshape(RMF_s(1,:),1,1,[]) + ...
                          reshape(axis_sample_coords(1,:),1,1,[]);
        sample_points_y = plane_coord_x.*reshape(RMF_r(2,:),1,1,[]) + ...
                          plane_coord_y.*reshape(RMF_s(2,:),1,1,[]) + ...
                          reshape(axis_sample_coords(2,:),1,1,[]);
        sample_points_z = plane_coord_x.*reshape(RMF_r(3,:),1,1,[]) + ...
                          plane_coord_y.*reshape(RMF_s(3,:),1,1,[]) + ...
                          reshape(axis_sample_coords(3,:),1,1,[]);
    
        cylin_extr = interp3(vol_y, vol_x, vol_z, vol, sample_points_y, sample_points_x, sample_points_z, 'cubic', mean(vol, "all"));
        cylin_extr = permute(cylin_extr, [3,2,1]);
        % cylin_extr: L, theta, r
    end
end