function [positions, qs] = filamentSubunitPoses(rise, twist, start_num, radius, smoothed_pts, RMF_r, RMF_s, RMF_t, base_pt)

    curv_len = size(smoothed_pts, 2) - 1;

    % max_order = ceil(curv_len/rise) - 1;
    % 
    % order = (0:max_order)';
    % start_index = 0:(start_num-1);
    % 
    % su_z = repmat(order * rise, 1, start_num);
    % su_theta = mod(start_index * (360/start_num) + order * twist, 360);

    latpt = helix.latticePoints([360/start_num;0],[twist;rise],[0,360-1e-5],[0,curv_len-1e-5],base_pt);
    su_theta = latpt(1,:);
    su_z = latpt(2,:);

    cyl_pts = [radius * cosd(su_theta(:)), radius * sind(su_theta(:)), su_z(:)]';
    
    base_pt_ind = floor(cyl_pts(3,:)) + 1;
    weight = cyl_pts(3,:) - floor(cyl_pts(3,:));

    base_pts = smoothed_pts(:, base_pt_ind) .* (1-weight) + smoothed_pts(:, base_pt_ind+1) .* weight;

    % RMF are x/y/z axis with respect to the base_pts
    base_x = RMF_r(:, base_pt_ind) .* (1-weight) + RMF_r(:, base_pt_ind+1) .* weight;
    base_y = RMF_s(:, base_pt_ind) .* (1-weight) + RMF_s(:, base_pt_ind+1) .* weight;
    base_z = RMF_t(:, base_pt_ind) .* (1-weight) + RMF_t(:, base_pt_ind+1) .* weight;

    positions = base_x.*cyl_pts(1,:) + base_y.*cyl_pts(2,:) + base_pts;
    
    su_local_x_axis = base_x .* cosd(su_theta(:)') + base_y .* sind(su_theta(:)');
    su_local_y_axis = -base_x .* sind(su_theta(:)') + base_y .* cosd(su_theta(:)');
    su_local_z_axis = base_z;

    qs = quaternion(permute(cat(3,su_local_x_axis, su_local_y_axis, su_local_z_axis), [1, 3, 2]), "rotmat", "point");

end