function [rf_rpv1,rf_rpv2,best_score,init_score,nn_score,mm_score] = ...
    rpvLocalRefine(score_map, rpv1, rpv2, sum_ind, ang_cand, len_cand)
    
    % Brief   : argmax the summation value of {score_map} at the lattice
    %           generated by {rpv1} and {rpv2}
    % Input   : unit of {len_dir_range} are pixel.
    
    if numel(sum_ind) == 1
        [sum_ind_x, sum_ind_y] = cencoord(sum_ind*[1,1]);
        sum_ind = [sum_ind_x(:), sum_ind_y(:)]';
    end

    init_score = scoreCalc(score_map, rpv1(:), rpv2(:), sum_ind);
    ft_unit = 1/size(score_map, 1);

    Nac = numel(ang_cand);
    Nlc = numel(len_cand);
    nn_score = -inf(Nac, Nac);
    nn_best_m1 = zeros(Nac, Nac);
    nn_best_m2 = zeros(Nac, Nac);

    current_max_s = -inf;
    current_best_n1 = 0;
    current_best_n2 = 0;
    mm_score = zeros(Nlc, Nlc);
    for n1 = 1:Nac
        for n2 = 1:Nac
            cand_rpv1 = [(rpv1(1)+ang_cand(n1))*ones(1, Nlc); rpv1(2)+len_cand*ft_unit];
            cand_rpv2 = [(rpv2(1)+ang_cand(n2))*ones(1, Nlc); rpv2(2)+len_cand*ft_unit];
            s = scoreCalc(score_map, cand_rpv1, cand_rpv2, sum_ind);
            s((cand_rpv1(1,:)==0 & abs(cand_rpv1(2,:))<1e-5) | (cand_rpv2(1,:)==0 & abs(cand_rpv2(2,:))<1e-5)) = -inf;
            s(abs(sind(vecsang(cand_rpv1,cand_rpv2))) < sind(3)) = -inf;
            [max_s, ind] = max(s, [], 'all');
            [m1, m2] = ind2sub([Nlc, Nlc], ind);
            if max_s > current_max_s
                current_max_s = max_s;
                current_best_n1 = n1;
                current_best_n2 = n2;
                mm_score = s;
            end
            nn_best_m1(n1, n2) = m1;
            nn_best_m2(n1, n2) = m2;
            nn_score(n1, n2) = max_s;
        end
    end
    
    best_score = current_max_s;
    best_n1 = current_best_n1;
    best_n2 = current_best_n2;
    rf_rpv1 = [rpv1(1)+ang_cand(best_n1); rpv1(2)+ft_unit*len_cand(nn_best_m1(best_n1, best_n2))];
    rf_rpv2 = [rpv2(1)+ang_cand(best_n2); rpv2(2)+ft_unit*len_cand(nn_best_m2(best_n1, best_n2))];

end

function score = scoreCalc(score_map, rpv1, rpv2, sum_ind)
    % {rpv1} and {rpv2} are 2xM and 2xK
    % {sum_ind} is 2xN
    % {score} is MxK

    map_sz = size(score_map);
    ft_unit = 1/size(score_map, 1);

    rpv1 = permute(rpv1, [3,2,4,1]); % [1,M,1,2]
    rpv2 = permute(rpv2, [3,4,2,1]); % [1,1,K,2]
    sum_ind = permute(sum_ind, [2,1]); % [N,2]

    pt_coords = rpv1 .* sum_ind(:,1) + rpv2 .* sum_ind(:,2); % [N,M,K,2];
    pt_subs = pt_coords;
    pt_subs(:,:,:,2) = pt_subs(:,:,:,2) / ft_unit;
    pt_subs = pt_subs + permute(cenpos(map_sz([2,1])), [3,4,1,2]);

    val = interp2(1:map_sz(2), 1:map_sz(1), score_map, pt_subs(:,:,:,1), pt_subs(:,:,:,2), ...
        'cubic', 0); % [N,M,K]
    score = squeeze(sum(val, 1)); % [M,K]

end

