function symm_vol = volHelicalSymmetrize(vol, msn, rise, twist, order)
%   msn: minminal start number, or the fold number of rotation symmetize
%   rise: in pixel
%   twist: in degree
%   order: order of implementing helical symmetry

    volsz = size(vol);
    [vx, vy, vz] = cencoord(volsz);
    vcd = [vx(:),vy(:),vz(:)];
    vxb = cencoord(volsz(1));
    vyb = cencoord(volsz(2));
    vzb = cencoord(volsz(3));

    T = @(v, od)(vol_sym_op(v, od, rise, twist, vcd, vxb, vyb, vzb));

    %%% Helical Symmetrize
    if order >= 1
        vp = vol;
        vn = vol;
        v_res = vol;
        bin_order = dec2bin(order) - '0';
        % for logord = 0:(floor(log2(order))-1)
        for ii = 1:numel(bin_order)
            op_ord = 2^(ii-1);
            vp = vp + T(vp, op_ord);
            vn = vn + T(vn, -op_ord);

            if bin_order(end+1-ii)
                offset_bin = bin_order * 0;
                offset_bin(1:end-ii) = bin_order(1:end-ii);
                offset_ord = bin2dec(char(offset_bin+'0'))+1;
                v_res = v_res + T(vp, offset_ord) + T(vn, -offset_ord);
            end

        end
        
        symm_vol = v_res / (2*order + 1);
    end

    %%% Helical Expansion
    symm_vol = helix.volHelicalExpand(symm_vol, rise, twist, order);
    % z_ord = round(vz / rise);
    % 
    % ref_z = vz - z_ord*rise;
    % ref_theta = -z_ord*twist;
    % ref_x = vx .* cosd(ref_theta) - vy .* sind(ref_theta);
    % ref_y = vy .* cosd(ref_theta) + vx .* sind(ref_theta);
    % 
    % z_msk = abs(vzb) < rise*(order + 0.5);
    % symm_vol = permute(z_msk, [3,2,1]).*reshape(interp3(vyb, vxb, vzb, symm_vol, ref_y(:), ref_x(:), ref_z(:), 'linear', 0), volsz);
    % 
    % clear ref_x;
    % clear ref_y;
    % clear ref_z;
    % clear ref_theta;

    %%% Rotational Symmetrize
    if msn > 1
        vp = symm_vol;
        rot_ang = 360/msn;
        for ir = 1:(msn-1)
            vcd_rt = quaternion([0,0,ir*rot_ang],'rotvecd').rotateframe(vcd);
            vol_rt = reshape(interp3(vyb, vxb, vzb, vp, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);
            symm_vol = symm_vol + vol_rt;
        end
        symm_vol = symm_vol ./ msn;
    end


end

function oped_v = vol_sym_op(vol, op_ord, rise, twist, vcd, vxb, vyb, vzb)
    vcd_rt = quaternion([0,0,twist*op_ord],'rotvecd').rotateframe(vcd) - [0,0,rise*op_ord];
    oped_v = reshape(interp3(vyb, vxb, vzb, vol, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), size(vol));
end
