function [x1,varargout] = cencoord(syssize, sysstep)
%CENCOORD - obtain the coordinate grid.
%   [X1,X2,...,XN] = CENCOORD(SYSSIZE), Xi is the i-th coordinate of a
%   coordinate system with size SYSSIZE.
%
%   [X1,X2,...,XN] = CENCOORD(SYSSIZE, SYSSTEP), Xi is the i-th coordinate
%   of a coordinate system with size SYSSIZE, the increasing step of Xi is
%   given by SYSSTEP. SYSSTEP can be scalar or vector with the same length
%   as SYSSIZE, if scalar, it will be extended to a vector by the scalar.
%
% Author: Mingtao Huang
% E-mail: hermitw57@outlook.com
% Last revision: 15-Feb-2022

%------------- BEGIN --------------

N = min(nargout, length(syssize));
syssize = syssize(:)';
if exist('sysstep', 'var')
    if numel(sysstep) == 1
        sysstep = ones(length(syssize), 1) * sysstep;
    end
end

if exist('sysstep', 'var')
    x1 = repmat(((1:syssize(1))'-cenpos(syssize, 1)).*sysstep(1), [1,syssize(2:end)]);
else
    x1 = repmat((1:syssize(1))'-cenpos(syssize, 1), [1,syssize(2:end)]);
end

varargout = cell(1, N-1);
for n = 2:N
    ref = reshape(1:syssize(n),[ones(1,n-1),syssize(n)]) - cenpos(syssize, n);
    if exist('sysstep', 'var')
        varargout{n-1} = repmat(ref.*sysstep(n), [syssize(1:n-1),1,syssize(n+1:end)]);
    else
        varargout{n-1} = repmat(ref, [syssize(1:n-1),1,syssize(n+1:end)]);
    end
end

%-------------- END ---------------

% update log:
%   2020-02-08: Construct this function and finish a basic version.
%   2021-04-22: Speed up.
%   2022-02-15: Add the SYSSTEP parameter to control the coordinates value.

