function [ra, rs, ra_recon] = ring_average(data, N, rrange)
% ring_average - Calculate the average DATA along radius
% [ra, rs, ra_recon] = ring_average(data, N, rrange)
%   data: the data to be averaged.
%   N: number of rings, L by default.
%   rrange: range of radius, [0,L-1] by default.

L = floor(max(size(data))/2);
if ~exist('N', 'var')
    N = L;
end
if ~exist('rrange', 'var')
    rrange = [0,L];
end

step = (rrange(2)-rrange(1))/N;
rs = ((-1:N)+0.5)*step+rrange(1);
total = zeros(1,N);
weights = zeros(1,N);

[x,y] = cencoord(size(data));
r = sqrt(x.^2+y.^2);
for n = 1:N
    msk = (r>rs(n))&(r<=rs(n+2));
    w = step - abs(r(msk)-rs(n+1));
    weights(n) = sum(w);
    total(n) = sum(w.*data(msk));
end

ra = total./weights;

if nargout >= 3
    ra_recon = zeros(size(data));
    
    ra_recon(r<=rs(2)) = ra(1);
    ra_recon(r>rs(end-1)) = ra(end);
    for n = 1:N-1
        msk = (r>rs(n+1))&(r<=rs(n+2));
        w = (r(msk)-rs(n+1))./step;
        ra_recon(msk) = w.*ra(n+1)+(1-w).*ra(n);
    end
end

rs = rs(2:end-1);

end